#include <iostream>
#include <cstdlib>
#include <fstream>
#include <sstream>
#include <string>
using namespace std;

void write_latex_odd_forall(int k, int l1) {
  string fname;
  ostringstream out;
  out << "ODD" << k <<"nplus"<<l1<<".tex";
  fname = out.str();

  ofstream fout;
  fout.open(fname);
  fout << "\\documentclass[12pt]{article}" << endl;
  fout << "\\usepackage{amsfonts,amssymb,amsthm}" << endl;
  fout << "\\begin{document}" << endl;
  fout << "\\newtheorem{theorem}{Theorem}[section]" << endl;
  fout << "\\begin{theorem}\\label{ODD_"<< k << "_" << l1 <<"}" << endl;
  fout << "For every natural number $n$, $" << k <<"n + "<< l1 << "$ is odd." << endl;
  fout << "\\end{theorem}" << endl;
  fout << "\\begin{proof}" << endl;
  fout << "We know that "<<k << " is even.  Also, we know that " << l1 << " is odd." << endl;
  // In both cases, we could provide evidence if we needed to... 
  fout << "Hence, $"<<k << "n$ must be even.  Since adding an even number to an odd number is always an odd number,  $" << k << "n+ " << l1 << "$ must be odd." << endl;
  fout << "\\end{proof}" << endl;
  fout << "\\end{document}" << endl;

}

void write_latex_even_forall(int k, int l1) {
  string fname;
  ostringstream out;
  out << "EVEN" << k <<"nplus"<<l1<<".tex";
  fname = out.str();

  ofstream fout;
  fout.open(fname);
  fout << "\\documentclass[12pt]{article}" << endl;
  fout << "\\usepackage{amsfonts,amssymb,amsthm}" << endl;
  fout << "\\begin{document}" << endl;
  fout << "\\newtheorem{theorem}{Theorem}[section]" << endl;
  fout << "\\begin{theorem}\\label{Even_"<< k << "_" << l1 <<"}" << endl;
  fout << "For every natural number $n$, $" << k <<"n + "<< l1 << "$ is even." << endl;
  fout << "\\end{theorem}" << endl;
  fout << "\\begin{proof}" << endl;
  fout << "We know that "<<k << " is even and " << l1 << " is even as well." << endl;
  // In both cases, we could provide evidence 
  fout << "Hence, $"<<k << "n$ must be even.  Since adding two even numbers always results in an even number,  $" << k << "n+ " << l1 << "$ must be even." << endl;
  fout << "\\end{proof}" << endl;
  fout << "\\end{document}" << endl;

}
void write_coq_even_forall(int k, int l1) {
  string fname;
  ostringstream out;
  out << "EVEN" << k <<"nplus"<<l1<<".v";
  fname = out.str();

  ofstream fout;
  fout.open(fname);

  fout << "Require Import Omega." << endl;
  fout << "(* Commment in COQ *)" << endl;
  fout << "(* Prove that this function is always even *)" << endl;
  fout << "(* using boolean values.  This is decidable, as opposed to the *)" << endl;
  fout << "(* the proposition.   Hence, its easier. *)" << endl;
  
  fout << "Lemma L"<<k<<"_"<<l1<<":forall n:nat, Nat.even("<<k<<"*n+"<<l1<<")=true." << endl;
  fout << "Proof." << endl;
  fout << "intros. "<< endl;
  

  fout << "assert (HY" << k <<":Nat.even("<<k<<")=true)." << endl;
  fout << "{" << endl;
  fout << "\tsimpl." << endl;
  fout << "\treflexivity." << endl;
  fout << "}" << endl;
  fout << "assert (HY" << l1 <<":Nat.even("<<l1<<")=true)." << endl;
  fout << "{" << endl;
  fout << "\tsimpl." << endl;
  fout << "\treflexivity." << endl;
  fout << "}" << endl;
  fout << "assert (HYE" << l1 <<":Nat.Even("<<l1<<"))." << endl;
  fout << "{" << endl;
  fout << "\tapply Nat.even_spec." << endl;
  fout << "\trewrite HY" << l1 << ";auto." << endl;
  fout << "}" << endl;
  fout << "assert (HYN: Nat.even("<<k << "*n)=true)." <<endl;
  fout << "{" << endl;
  fout << "\t rewrite Nat.even_mul." << endl;
  fout << "\t rewrite HY" << k << "." << endl;
  fout << "\t simpl. "<< endl;
  fout << "\t reflexivity." << endl;
  fout << "}" << endl;
  fout << "rewrite Nat.even_add_even." << endl;
  fout << "rewrite HYN." << endl;
  fout << "reflexivity." << endl;
  fout << "apply  HYE" << l1 << "." << endl;
  fout << "Qed." << endl;
  fout << "(* Commment in COQ *)" << endl;
  fout << "(* Prove that this function is always even *)" << endl;
  fout << "(* in Propositional Form.  Use the Boolean Lemma above. *)" << endl;
  
  fout << "Theorem T"<<k<<"_"<<l1<<":forall n:nat, Nat.Even("<<k<<"*n+"<<l1<<")." << endl;
  fout << "Proof." << endl;
  fout << "intros. "<< endl;
  fout << "apply Nat.even_spec." << endl;
  fout << "apply L"<<k<<"_"<<l1 << "." <<endl;
  fout << "Qed." << endl;
  
  

}
void write_coq_odd_forall(int k, int l1) {
  string fname;
  ostringstream out;
  out << "ODD" << k <<"nplus"<<l1<<".v";
  fname = out.str();

  ofstream fout;
  fout.open(fname);

  fout << "Require Import Omega." << endl;
  fout << "(* Commment in COQ *)" << endl;
  fout << "(* Prove that this function is always odd *)" << endl;
  fout << "(* using boolean values.  This is decidable, as opposed to the *)" << endl;
  fout << "(* the proposition.   Hence, its easier. *)" << endl;
  
  fout << "Lemma L"<<k<<"_"<<l1<<":forall n:nat, Nat.odd("<<k<<"*n+"<<l1<<")=true." << endl;
  fout << "Proof." << endl;
  fout << "intros. "<< endl;
  

  fout << "assert (HY" << k <<":Nat.even("<<k<<")=true)." << endl;
  fout << "{" << endl;
  fout << "\tsimpl." << endl;
  fout << "\treflexivity." << endl;
  fout << "}" << endl;
  fout << "assert (HY" << l1 <<":Nat.odd("<<l1<<")=true)." << endl;
  fout << "{" << endl;
  fout << "\t unfold Nat.odd." << endl;
  fout << "\tsimpl." << endl;
  fout << "\treflexivity." << endl;
  fout << "}" << endl;
  fout << "assert (HYN: Nat.even("<<k << "*n)=true)." <<endl;
  fout << "{" << endl;
  fout << "\t rewrite Nat.even_mul." << endl;
  fout << "\t rewrite HY" << k << "." << endl;
  fout << "\t simpl. "<< endl;
  fout << "\t reflexivity." << endl;
  fout << "}" << endl;
  fout << "assert (HYE" << k <<":Nat.Even("<<k<<"*n))." << endl;
  fout << "{" << endl;
  fout << "\tapply Nat.even_spec." << endl;
  fout << "\trewrite HYN;auto." << endl;
  fout << "}" << endl;
  fout << "assert (HR:"<<l1<<"+" <<k <<"*n = " << k << "*n+ "<< l1<<")."<<endl;
  fout << "{" << endl;
  fout << "\t omega. " << endl;
  fout << "}" << endl;
  fout << "rewrite <-HR." << endl;
  fout << "rewrite Nat.odd_add_even." << endl;
  fout << "rewrite HY" << l1<<"." << endl;
  fout << "reflexivity." << endl;
  fout << "apply HYE" <<k<<"." << endl;
  fout << "Qed." << endl;
  fout << "(* Commment in COQ *)" << endl;
  fout << "(* Prove that this function is always odd *)" << endl;
  fout << "(* in Propositional Form.  Use the Boolean Lemma above. *)" << endl;
  
  fout << "Theorem T"<<k<<"_"<<l1<<":forall n:nat, Nat.Odd("<<k<<"*n+"<<l1<<")." << endl;
  fout << "Proof." << endl;
  fout << "intros. "<< endl;
  fout << "apply Nat.odd_spec." << endl;
  fout << "apply L"<<k<<"_"<<l1 << "." <<endl;
  fout << "Qed." << endl;
  
  

}







int main(int argc, char *argv[]) {

  for (int i=0;i<100;i++) {
    int j = rand();

    int k =j/(RAND_MAX/100);

    int l = rand();

    int l1 = l/(RAND_MAX/100);

    cout << " k = " << k << " l1= " << l1 << endl;

    if (k%2 == 0) {
      if (l1% 2 == 0) {
	
	cout << "Forall n in NAT, " << k << "n + " << l1 << " is EVEN" << endl;
	write_latex_even_forall(k,l1);
	write_coq_even_forall(k,l1);
      } else {
	cout << "Forall n in NAT, " << k << "n + " << l1 << " is ODD" << endl;
	write_latex_odd_forall(k,l1);
	write_coq_odd_forall(k,l1);
      }
    } else {
      cout << "There exists a n1, n2 in NAT such that " << endl;
      cout << k << "n1 + " << l1 << " is EVEN and " << endl;
      cout << k << "n2 + " << l1 << " is ODD " << endl;
    }
    
  }
}
